{* Virtual.PAS                                                              *}
{* Copyright (c) Genus Microprogramming, Inc. 1988-89  All Rights Reserved. *}

{****************************************************************************

  This program demonstrates the usage of a virtual window. It is an example
  Pascal routine for the PCX Programmer's Toolkit.


  Borland Turbo Pascal 4.0/5.0            Programmer: Chris Howard  4/15/89

*****************************************************************************}

Program Virtual; 

{ Include the PCX Toolkit Unit, and any other units needed }
uses pcx_tp,crt;

{ Globals }
const
  pcxtype        : integer = pcxVGA_11;
  pcximage       = 'Virtual.PCX';

  bufmax         = 20000;

var

  vi             : array[1..2] of PCXVINFO;
  header         : PCXHEADER;

  vfree,vreq     : longint;
  vptr           : longint;

  width,depth    : integer;
  wx2,wy2        : integer;

  retcode        : integer;
  key            : char;

  buffer         : array[1..bufmax] of byte;

{**********}

begin

  { Display program header }
  writeln;
  writeln('͸');
  writeln(' Virtual: Example Virtual Pascal Program    PCX Programmer''s Toolkit 3.50 ');
  writeln(' Copyright (c) Genus Microprogramming, Inc. 1988-89  All Rights Reserved. ');
  writeln(';');
  writeln;

  { Get a key, to begin }
  writeln('Press a key to run . . .');
  key := ReadKey;

  { Allocate a larter toolkit buffer, to speed up file and display speeds }
  retcode := pcxSetBuffer(@buffer,bufmax);

  { Check if we have expanded memory, and display }
  if (pcxEMInstalled = pcxSUCCESS) then begin
    vfree := pcxVirtualFree(pcxEMM);
    writeln('Free EMM: ',vfree);
  end; {of if}

  { Display the amount of conventional free memory }
  vfree := pcxVirtualFree(pcxCMM);
  writeln('Free CMM: ',vfree);

  { Look at the current hardware, to find a B/W mode }
  retcode := pcxVideoInfo(@vi);

  case vi[1].adapter of
    viCGA: pcxtype := pcxCGA_6;
    viEGA: begin
             if (vi[1].display = viMDAdisp) then
               pcxtype := pcxEGA_F
             else
               pcxtype := pcxEGA_10;
           end;
    viVGA: pcxtype := pcxVGA_11;
    viHGC: pcxtype := pcxHERC;
    else   pcxtype := -1;
  end; {of case}

  { If we found a mode, continue }
  if (pcxtype <> -1) then begin

    { Get the file header }
    retcode := pcxGetFileHeader(pcximage,@header);
    if (retcode = pcxSUCCESS) then begin

      { Get the image size }
      width := header.x2-header.x1+1;
      depth := header.y2-header.y1+1;

      { Get the virtual memory required }
      vreq := pcxVirtualSize(pcxtype,width,depth);

      { Display it }
      writeln('Reqd CMM: ',vreq);
      writeln;

      { Wait for a key }
      writeln('Press a key to continue . . .');
      key := ReadKey;

      { Do we have enough? }
      if (vreq < vfree) then begin

        { Yes, so create the virtual buffer }
        retcode := pcxCreateVirtual(pcxCMM,@vptr,pcxtype,width,depth);
        if (retcode = pcxSUCCESS) then begin

          { The buffer was created, so load the file }
          retcode := pcxFileVirtual(pcximage,vptr);
          if (retcode = pcxSUCCESS) then begin

            { We loaded, so for example define a window 160x160 }
            { (remember: 0 to 159 = 160)                        }
            wx2 := 159;
            wy2 := 159;

            { NOTE: NEVER DEFINE A WINDOW LARGER THAN THE IMAGE FOR SCROLLING! }
            if (wx2 > width-1) then
              wx2 := width-1;
            if (wy2 > depth-1) then
              wy2 := depth-1;

            { Set the display type and mode }
            retcode := pcxSetDisplay(pcxtype);
            retcode := pcxSetMode(pcxGRAPHICS);

            { Allow scrolling of the image in the window }
            retcode := pcxVirtualScroll(vptr,width,depth,0,0,wx2,wy2,0);

            { Get back to text mode }
            retcode := pcxSetMode(pcxTEXT);

            { Were we successful? }
            if (retcode <> pcxSUCCESS) then
              writeln('pcxVirtualScroll Error: ',retcode);

          end
          else begin
            { Error loading file to virtual buffer }
            writeln('Error loading file: ',retcode);
          end; {of else}

          { Destroy the virtual buffer }
          retcode := pcxDestroyVirtual(vptr);

        end
        else begin
          { Error creating virtual buffer }
          writeln('Error creating virtual buffer: ',retcode);
        end; {of else}

      end 
      else begin
        { Not enough free memory }
        writeln('Not enough free memory . . .');
      end;

    end
    else begin
      { Could not get image header }
      writeln('Could not load image header: ',retcode);
    end;

  end
  else begin
    { We could not find a good mode }
    writeln('Could not find a good mode type . . .');
  end; {of else}

end. { end of main }

